@@ -59,6 +59,9 @@ class TypeHintError(Exception): pass
 _mid = lambda x: x[1] if len(x) > 1 else x[0]
 _rawname = lambda s: _mid(str(s).split("'")).split('.')[-1]
 
+def equals(x, y):
+    return (isinstance(x, type(y)) or isinstance(y, type(x))) and x == y
+
 def inheritable(x):
     """
     inheritable(x) -> bool
@@ -124,13 +127,19 @@ def isitertype(x):
         >>> isitertype(str)
         False
     """
+    if isinstance(x, str):
+        local_vars = get_environ_vars()
+        local_vars.update(locals())
+        locals().update(local_vars.simplify())
+        try: x = eval(x)
+        except: return False
     if isinstance(x, Type):
         return all([isitertype(i) for i in x.types])
     if isinstance(x, type):
         try: return iterable(x())
         except Exception: return False
     if callable(x):
-        try: return x('iterable check') == ''
+        try: return equals(x('iterable check'), '')
         except Exception: return False
     raise SystemError("Conflict between functions 'isatype' and 'isitertype', please contact the developer for more information. ")
 
@@ -165,8 +174,8 @@ def isarray(x):
         >>> isarray([1, 2])
         False
     """
-    if isinstance(x, str) and x == 'iterable check': return ''
-    if not isatype(x) and 'shape' in dir(x): return True
+    if equals(x, 'iterable check'): return ''
+    if not isatype(x) and hasattr(x, 'shape'): return True
     return False
 
 def isdtype(x):
@@ -216,6 +225,7 @@ def isatype(x):
     """
     if x is None: return True
     if isinstance(x, type): return True
+    if isinstance(x, str): return True
     if callable(x):
         name = _get_func_name(x)
         if not name: return False
@@ -245,7 +255,7 @@ def typename(x):
     except: pass
     return name
 
-def isoftype(x, xtype, environ_func = None):
+def isoftype(x, xtype):
     """
     isoftype(x, xtype) -> bool
 
@@ -270,7 +280,7 @@ def isoftype(x, xtype, environ_func = None):
         False
     """
     if isinstance(xtype, str):
-        local_vars = get_environ_vars(isoftype if environ_func is None else environ_func)
+        local_vars = get_environ_vars()
         local_vars.update(locals())
         locals().update(local_vars.simplify())
         try: xtype = eval(xtype)
@@ -298,7 +308,7 @@ def isoftype(x, xtype, environ_func = None):
 
 def isclassmethod(x):
     if not callable(x): return False
-    if '__qualname__' not in dir(x): return False
+    if hasattr(x, '__qualname__'): return False
     p = x.__qualname__.split('.')
     if len(p) <= 1: return False
     try:
@@ -314,8 +324,8 @@ class _strIO:
     def split(self, c=None): return self._str_.split(c)
 
 def _getDeclaration(func):
-    if have_inspect:
-        dec = [l for l in inspect.getsource(func).split('\n') if l.strip() and not l.strip().startswith('@')][0].strip().strip(':')
+    if have_inspect and '__file__' in func.__globals__:
+        dec = next(l for l in inspect.getsource(func).split('\n') if l.strip() and not l.strip().startswith('@')).strip().rstrip(':')
     else:
         ss = _strIO()
         oldout = sys.stdout
@@ -348,7 +358,7 @@ def list_type_args(func):
             pre = args[:1]
             args = args[1:]
         else: pre = tuple()
-        if len(args) == 1 and type(args[0]) in (list, tuple, set):
+        if len(args) == 1 and type(args[0]) in (list, tuple):
             for item in args[0]:
                 if not isatype(item):
                     raise TypeHintError("Unsupported types of arguments for " +
@@ -401,14 +411,14 @@ class Type(type):
 
     def __getitem__(self, i=None):
         if i == 0 or i is None: i = tuple()
-        if isatype(i): return self@i
-        if isinstance(i, slice): return self@{i.start:i.stop}
-        if isoftype(i, int): i = (i,)
-        elif isoftype(i, sequence): pass
+        if isoftype(i, sequence): pass
+        elif isatype(i): return self@i
+        elif isinstance(i, slice): return self@{i.start:i.stop}
+        elif isoftype(i, int): i = (i,)
         else: raise TypeHintError("Wrong size specifier. ")
         if all([x is None or isitertype(x) for x in self.types]):
             if not all([isoftype(x, int) for x in i]):
-                if len(i) == 1 and isinstance(i, list): return self@i
+                if isinstance(i, list): return self@i
                 if all([isinstance(x, slice) for x in i]): return self@{x.start:x.stop for x in i}
                 if all([isinstance(x, str) or isatype(x) for x in i]): return self@Type(*i)
                 raise TypeHintError("Wrong size specifier. ")
@@ -426,8 +436,8 @@ class Type(type):
         if len(l) == 1: return self[l[0]]
         return self[tuple(l)]
 
-    def __or__(self, other): return Type(self, other, name=f"{self.rawname}|{other.rawname}")
-    def __ror__(self, other): return Type(other, self, name=f"{other.rawname}|{self.rawname}")
+    def __or__(self, other): return Type(self, other, name=f"{self.rawname}|{other.rawname if isinstance(other, Type) else _rawname(other)}")
+    def __ror__(self, other): return Type(other, self, name=f"{other.rawname if isinstance(other, Type) else _rawname(other)}|{self.rawname}")
     def __and__(self, other): return Type(~Type(~self, ~other), name=f"{self.rawname}&{other.rawname}")
     def __rand__(self, other): return Type(~Type(~other, ~self), name=f"{other.rawname}&{self.rawname}")
 
@@ -499,7 +509,7 @@ class Type(type):
         if isoftype(arg, self.types):
             if len(self.shape) == 0: pass
             elif len(self.shape) == 1 and (len(arg) == self.shape[0] or self.shape[0] in (None, -1)): pass
-            elif len(self.shape) > 1 and 'shape' in dir(arg) and \
+            elif len(self.shape) > 1 and hasattr(arg, 'shape') and \
                 all([a==b or b in (None, -1) for a, b in zip(arg.shape, self.shape)]): pass
             else: return false
             if self.itemtypes != None:
@@ -560,12 +570,12 @@ Tuple = T(tuple)
 Class = T(type, name="Class")
 Callable = T(callable)
 Function = T(type(iterable))
-Method = T(type(_strIO().split), type("".split), type(Int.__str__), name="Method")
+Method = T(type(_strIO().split), type("".split), type(Int.__str__), type(staticmethod(lambda:None)), type(classmethod(lambda:None)), name="Method")
 Lambda = T(type(lambda: None), name="Lambda")
 Functional = T(Function, Method, Lambda, name="FunctionType")
 Real = T(float, int, name="Real")
-sequence = [tuple, list, set]
-Sequence = T(tuple, list, set, name="Sequence")
+sequence = [tuple, list]
+Sequence = T(tuple, list, name="Sequence")
 null = type(None)
 Null = T(null)
 real = [int, float]
@@ -609,7 +619,7 @@ class ArrayType(Type):
         except ImportError: pass
 
 Array = ArrayType(name="Array")
-Iterable = T(Array, Sequence, dict, name="Iterable")
+Iterable = T(Array, Sequence, set, dict, name="Iterable")
 
 class ScalarType(Type):
     def __call__(self, x):
@@ -654,7 +664,7 @@ def getArgs(func, *args, **kwargs):
         if getTypes: allargs[var] = type(func.__defaults__[idefault])
         else: allargs[var] = func.__defaults__[idefault]
     if eargs: allargs[eargs] = tuple(inputargs); inputargs = []
-    if ekwargs: allargs[ekwargs] = inputargs.pop(0) if len(inputargs) == 1 else kwargs; kwargs = {}
+    if ekwargs: allargs[ekwargs] = inputargs[0] if len(inputargs) == 1 and isinstance(inputargs[0], dict) else kwargs; kwargs = {}
     for addkwarg in extargs:
         if addkwarg in (eargs, ekwargs): continue
         if addkwarg in kwargs:
@@ -674,6 +684,61 @@ def getArgs(func, *args, **kwargs):
 
 @decorator
 def params(*types, run=True, **kwtypes):
+    if len(types) == 1 and len(kwtypes) == 0 and Functional(types[0]): return params()(types[0])
+    @decorator
+    def induced_decorator(func):
+        declaration = _getDeclaration(func)
+        eargs = ''.join(re.findall(r"[^*]\*{1} *(\w+)\b", declaration))
+        ekwargs = ''.join(re.findall(r"[^*]\*{2} *(\w+)\b", declaration))
+        depth = []
+        d = 0;
+        for c in declaration:
+            if c in '([{': d += 1; depth.append(d)
+            elif c in '}])': depth.append(d); d -= 1
+            else: depth.append(d)
+            if d == 0: break
+        declaration = ''.join(c for x, c in zip(depth, declaration) if x == 1)
+        declaration = declaration.strip('()')
+        declaration = ','.join(p.split(':')[0].split('=')[0].strip() + ("='__default__'" if re.search('[^=]=[^=]', p) else '') for p in declaration.split(','))
+        exec(f"def tmp({declaration}): return locals()")
+        fetch = eval('tmp')
+        if len(types) == len(kwtypes) == 0:
+            annotations = func.__annotations__
+            if 'return' in annotations:
+                rtype = annotations.pop('return')
+            else: rtype = None
+        else:
+            if '__return__' in kwtypes:
+                rtype = kwtypes.pop('__return__')
+            else: rtype = None
+            annotations = fetch(*types, **kwtypes)
+        def wrapper_func(*args, **kwargs):
+            try: values = fetch(*args, **kwargs)
+            except Exception as e: raise TypeHintError(str(e))
+            for arg in values:
+                if equals(values[arg], "__default__") or arg not in annotations: continue
+                if arg == eargs:
+                    if Sequence(annotations[arg]) and len(annotations[arg]) == len(values[arg]):
+                        if not all(isoftype(v, t) for v, t in zip(values[arg], annotations[arg])): break
+                    if any(not isoftype(x, annotations[arg]) for x in values[arg]): break
+                elif arg == ekwargs:
+                    if Dict(annotations[args]):
+                        if not all(isoftype(values[arg][k], annotations[arg][k]) for k in annotations[args]): break
+                    if any(not isoftype(x, annotations[arg]) for x in values[arg].values()): break
+                elif not isoftype(values[arg], annotations[arg]): break
+            else:
+                if run:
+                    r = func(*args, **kwargs)
+                    if isoftype(r, rtype): return r
+                    else: raise TypeHintError(f"{_get_func_name(func)}() has return value of wrong type. Expect type {repr(rtype)} but got {repr(r)}.")
+                else: return None
+            raise TypeHintError(f"{_get_func_name(func)}() has argument {arg} of wrong type. Expect type {repr(annotations[arg])} but got {repr(values[arg])}.")
+        return wrapper_func
+    return induced_decorator
+        
+
+@decorator
+def params_old(*types, run=True, **kwtypes):
     israw = len(kwtypes) == 0 and len(types) == 1 and \
             callable(types[0]) and not isatype(types[0])
     @decorator
@@ -719,12 +784,12 @@ def params(*types, run=True, **kwtypes):
                         _annotations[eargs] = Type(list, tuple)@_annotations[eargs]
             for arg in _values:
                 if arg in _annotations:
-                    if isoftype(_values[arg], _annotations[arg], environ_func=func): continue
+                    if isoftype(_values[arg], _annotations[arg]): continue
                     break
             else:
                 if run:
                     retval = func(*args, **kwargs)
-                    if 'return' not in _annotations or isoftype(retval, _annotations['return'], environ_func=func): return retval
+                    if 'return' not in _annotations or isoftype(retval, _annotations['return']): return retval
                     raise TypeHintError(_get_func_name(func) + "() returns an invalid value.")
                 else: return None
             raise TypeHintError(_get_func_name(func) + "() has argument " + arg + " of wrong type. \
@@ -764,11 +829,11 @@ if __name__ == "__main__":
         Dict[2]@{(Int, str):Real}
     ))
 
-    @params(Func, Int, +Int, __return__ = Real[2])
+    @params(Functional, Int, +Int, __return__ = Real[2])
     def test_func(a, b=2, *k):
         return k
 
-    print(isclassmethod(extendable))
+    print(isclassmethod(inheritable))
     print(isclassmethod(Float.__len__))
     print(isclassmethod(Type.__len__))
 
@@ -783,4 +848,4 @@ if __name__ == "__main__":
     print(List[2]@[int, Tuple[3]])
     print((List[2]@[int, Tuple[3]])([1, (2.0, 3, 'a')]))
     print(isoftype(0.1, Int))
-    test_func(lambda x: 1, 3, 4, 5)
+    print(test_func(lambda x: 1, 3, 4, 5))
